import {AfterViewInit, ChangeDetectorRef, Component, Input, OnDestroy, ViewChild} from "@angular/core";
import {CreTable} from "../../shared/components/tables/tables";
import {MixMaterialDto, sortMixMaterialsDto} from "../../shared/model/recipe.model";
import {Observable, Subject} from "rxjs";
import {Material} from "../../shared/model/material.model";
import {FormControl, Validators} from "@angular/forms";
import {takeUntil} from "rxjs/operators";
import {CreInputEntry} from "../../shared/components/inputs/inputs";

@Component({
  selector: 'cre-mix-materials-form',
  templateUrl: 'materials-form.html'
})
export class MixMaterialsForm implements AfterViewInit, OnDestroy {
  @ViewChild(CreTable) table: CreTable<MixMaterialDto>

  @Input() materials: Observable<Material[]>

  mixMaterials: MixMaterialDto[] = []
  columns = ['position', 'positionButtons', 'material', 'quantity', 'endButton']

  private _allMaterials: Material[]
  private _controls: ControlsByPosition[] = []
  private _availableMaterialsEntries: MaterialEntriesByPosition[] = []
  private _destroy$ = new Subject<boolean>()

  constructor(
    private cdRef: ChangeDetectorRef
  ) {
  }

  ngAfterViewInit() {
    this.materials.subscribe({
      next: materials => {
        this._allMaterials = materials

        this.addRow()
        this.cdRef.detectChanges()
      }
    })
  }

  ngOnDestroy() {
    this._destroy$.next(true)
    this._destroy$.complete()
  }

  addRow() {
    const position = this.nextPosition;
    const mixMaterial = new MixMaterialDto(0, 0, false, position);

    const materialIdControl = new FormControl(null, Validators.required)
    const quantityControl = new FormControl(null, Validators.required)

    materialIdControl.valueChanges
      .pipe(takeUntil(this._destroy$))
      .subscribe({
        next: materialId => {
          this.mixMaterials.filter(x => x.materialId === mixMaterial.materialId)[0].materialId = materialId
          this.refreshAvailableMaterials()
        }
      })

    this._controls.push({
      position,
      controls: {
        materialId: materialIdControl,
        quantity: quantityControl
      }
    })
    this._availableMaterialsEntries.push({
      position,
      entries: this.filterMaterials(mixMaterial)
    })
    this.mixMaterials.push(mixMaterial)
    this.table.renderRows()
  }

  removeRow(mixMaterial: MixMaterialDto) {
    this.mixMaterials = this.mixMaterials.filter(x => x.position != mixMaterial.position)

    for (let position = mixMaterial.position + 1; position < this.mixMaterials.length; position++) {
      this.updatePosition(this.getMixMaterialByPosition(position), position - 1, false)
    }
  }

  updatePosition(mixMaterial: MixMaterialDto, newPosition: number, switchPositions = true) {
    const currentPosition = mixMaterial.position

    this.getControlsByPosition(currentPosition).position = newPosition
    this.getMaterialEntriesByPosition(currentPosition).position = newPosition

    if (switchPositions) {
      this.updatePosition(this.getMixMaterialByPosition(newPosition), currentPosition, false)
    }

    mixMaterial.position = newPosition
    this.sortTable()
  }

  getControls(position: number): MixMaterialControls {
    return this.getControlsByPosition(position).controls
  }

  getAvailableMaterialEntries(position: number): CreInputEntry[] {
    return this.getMaterialEntriesByPosition(position).entries
  }

  get materialCount(): number {
    return this._allMaterials ? this._allMaterials.length : 0;
  }

  private get nextPosition(): number {
    return this.mixMaterials.length
  }

  private getMixMaterialByPosition(position: number): MixMaterialDto {
    return this.mixMaterials.filter(x => x.position === position)[0]
  }

  private getControlsByPosition(position: number): ControlsByPosition {
    return this._controls.filter(control => control.position === position)[0]
  }

  private getMaterialEntriesByPosition(position: number): MaterialEntriesByPosition {
    return this._availableMaterialsEntries.filter(control => control.position === position)[0]
  }

  private refreshAvailableMaterials() {
    this.mixMaterials
      .sort((a, b) => a.position - b.position)
      .forEach(mixMaterial => {
        this.getMaterialEntriesByPosition(mixMaterial.position).entries = this.filterMaterials(mixMaterial)
      })
  }

  private sortTable() {
    this.mixMaterials = sortMixMaterialsDto(this.mixMaterials)
    this.table.renderRows()
  }

  private filterMaterials(mixMaterial: MixMaterialDto): CreInputEntry[] {
    return this._allMaterials
      .filter(material => mixMaterial.materialId === material.id || this.mixMaterials.filter(mm => mm.materialId === material.id).length === 0)
      .map(material => new CreInputEntry(material.id, material.name))
  }
}

interface MixMaterialControls {
  materialId: FormControl
  quantity: FormControl
}

interface ControlsByPosition {
  position: number
  controls: MixMaterialControls
}

interface MaterialEntriesByPosition {
  position: number
  entries: CreInputEntry[]
}
